<?php namespace Modules\Forum\Database\Migrations;
/**
 * @author			Aby Dahana
 * @profile			abydahana.github.io
 * @website			www.aksaracms.com
 * @copyright		(c) 2021 - Aksara Laboratory
 * @since			version 4.2.8
 */
use CodeIgniter\Database\Migration;

class Installer extends Migration
{
	public function up()
	{
		// downgrade migration to delete the previous created tables
		$this->down();
		
		$this->forge->addField
		(
			array
			(
				'category_id' => array
				(
					'type' => 'bigint',
					'constraint' => 22,
					'unsigned' => true,
					'auto_increment' => true,
					'null' => false
				),
				'category_slug' => array
				(
					'type' => 'varchar',
					'constraint' => 255,
					'null' => false
				),
				'category_title' => array
				(
					'type' => 'varchar',
					'constraint' => 255,
					'null' => false
				),
				'category_description' => array
				(
					'type' => (in_array($this->db->DBDriver, array('Postgre', 'SQLSRV')) ? 'text' : 'tinytext'),
					'null' => false
				),
				'category_image' => array
				(
					'type' => 'varchar',
					'constraint' => 255,
					'null' => false
				),
				'status' => array
				(
					'type' => 'tinyint',
					'constraint' => 1,
					'default' => '0',
					'null' => false
				)
			)
		);
		$this->forge->addKey('category_id', true, true);
		$this->forge->addKey('category_slug', false, true);
		$this->forge->addKey('category_title', false, true);
		$this->forge->createTable('forum__categories');
		
		$this->forge->addField
		(
			array
			(
				'topic_id' => array
				(
					'type' => 'bigint',
					'constraint' => 22,
					'unsigned' => true,
					'auto_increment' => true,
					'null' => false
				),
				'category_id' => array
				(
					'type' => 'bigint',
					'constraint' => 22,
					'unsigned' => true,
					'null' => false
				),
				'user_id' => array
				(
					'type' => 'bigint',
					'constraint' => 22,
					'unsigned' => true,
					'null' => false
				),
				'topic_slug' => array
				(
					'type' => 'varchar',
					'constraint' => 255,
					'null' => false
				),
				'topic_title' => array
				(
					'type' => 'varchar',
					'constraint' => 255,
					'null' => false
				),
				'topic_description' => array
				(
					'type' => (in_array($this->db->DBDriver, array('Postgre', 'SQLSRV')) ? 'text' : 'longtext'),
					'null' => false
				),
				'topic_attachment' => array
				(
					'type' => (in_array($this->db->DBDriver, array('Postgre', 'SQLSRV')) ? 'text' : 'longtext'),
					'null' => false
				),
				'created_at' => array
				(
					'type' => 'timestamp',
					'null' => false
				),
				'updated_at' => array
				(
					'type' => 'timestamp',
					'null' => true
				),
				'status' => array
				(
					'type' => 'tinyint',
					'constraint' => 1,
					'default' => '0',
					'null' => false
				)
			)
		);
		$this->forge->addKey('topic_id', true, true);
		$this->forge->addKey('category_id', false, false);
		$this->forge->addKey('user_id', false, false);
		$this->forge->addForeignKey('category_id', 'forum__categories', 'category_id', ($this->db->DBDriver == 'SQLSRV' ? 'NO ACTION' : 'CASCADE'), ($this->db->DBDriver == 'SQLSRV' ? 'NO ACTION' : 'CASCADE'));
		$this->forge->addForeignKey('user_id', 'app__users', 'user_id', ($this->db->DBDriver == 'SQLSRV' ? 'NO ACTION' : 'CASCADE'), ($this->db->DBDriver == 'SQLSRV' ? 'NO ACTION' : 'CASCADE'));
		$this->forge->createTable('forum__topics');
		
		$this->forge->addField
		(
			array
			(
				'topic_id' => array
				(
					'type' => 'bigint',
					'constraint' => 22,
					'unsigned' => true,
					'null' => false
				),
				'history' => array
				(
					'type' => (in_array($this->db->DBDriver, array('Postgre', 'SQLSRV')) ? 'text' : 'longtext'),
					'null' => false
				),
				'attachment' => array
				(
					'type' => (in_array($this->db->DBDriver, array('Postgre', 'SQLSRV')) ? 'text' : 'longtext'),
					'null' => false
				),
				'timestamp' => array
				(
					'type' => 'timestamp'
				)
			)
		);
		
		$this->forge->addKey('topic_id', false, false);
		$this->forge->addForeignKey('topic_id', 'forum__topics', 'topic_id', ($this->db->DBDriver == 'SQLSRV' ? 'NO ACTION' : 'CASCADE'), ($this->db->DBDriver == 'SQLSRV' ? 'NO ACTION' : 'CASCADE'));
		$this->forge->createTable('forum__topics_history');
		
		$this->forge->addField
		(
			array
			(
				'topic_id' => array
				(
					'type' => 'bigint',
					'constraint' => 22,
					'unsigned' => true,
					'null' => false
				),
				'user_id' => array
				(
					'type' => 'bigint',
					'constraint' => 22,
					'unsigned' => true,
					'null' => false
				),
				'reason' => array
				(
					'type' => (in_array($this->db->DBDriver, array('Postgre', 'SQLSRV')) ? 'text' : 'tinytext'),
					'null' => false
				),
				'timestamp' => array
				(
					'type' => 'timestamp'
				)
			)
		);
		
		$this->forge->addKey('topic_id', false, false);
		$this->forge->addForeignKey('topic_id', 'forum__topics', 'topic_id', ($this->db->DBDriver == 'SQLSRV' ? 'NO ACTION' : 'CASCADE'), ($this->db->DBDriver == 'SQLSRV' ? 'NO ACTION' : 'CASCADE'));
		$this->forge->addForeignKey('user_id', 'app__users', 'user_id', ($this->db->DBDriver == 'SQLSRV' ? 'NO ACTION' : 'CASCADE'), ($this->db->DBDriver == 'SQLSRV' ? 'NO ACTION' : 'CASCADE'));
		$this->forge->createTable('forum__topics_report');
	}
	
	public function down()
	{
		// downgrade migration to delete the previous created tables
		$tables										= array('forum__replies_report', 'forum__replies_history', 'forum__replies_likes', 'forum__topics_report', 'forum__topics_history', 'forum__topics_likes', 'forum__replies', 'forum__topics', 'forum__categories');
		
		foreach($table as $key => $val)
		{
			if($this->db->tableExists($val))
			{
				$this->forge->dropTable($val);
			}
		}
	}
}
